/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * ----------------------------------------------------------------------
 * enhanced by Aley Keprt, 01d.05m.1998
 * (just a short echo displayed on exit, color in DOS version)
 * ----------------------------------------------------------------------
 *
 * (start/stop)watch emulator timing code (c) 1994 Ian Collier.
 *
 * File util.c - contain various common utility functions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifdef VGA_DISPLAY
 #include <conio.h>
#else
 #define cprintf printf
 #define textattr(x)
#endif

#ifdef US_KEYBOARD
 #define KEYBMODE "US"
#else
 #define KEYBMODE "UK"
#endif


#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <ctype.h>

#include "z80.h"
#include "mem.h"
#include "util.h"
#include "resource.h"

/* Emulator timing variables */

static struct timeval watch;
static unsigned long millisec;
static int stopped;

static FILE *logfp = NULL;

/* list of functions to be called when shutting down */
static struct on_quit_rec {
	void (*f)();
	struct on_quit_rec *next;
} *on_quit_list = 0;


/* Open a file, checking each element in the colon-separated resource
 * 'lib_dir', unless the filename is an absolute path or relative to the
 * current or parent directory.  Behaves like fopen() in other respects.
 */
FILE *
open_simcoupe_file(char *name, char *amode)
{
	FILE *fp = 0;
	char *buf;
	char *path, *element;

	if (!name || !name[0])
		return NULL;

	if (name[0] == '/' || strncmp(name, "./", 2) == 0 ||
			strncmp(name, "../", 3) == 0)
		return(fopen(name, amode));

	path = (char *) malloc(strlen(GetCfg(lib_dir)) + 1);
	strcpy(path, GetCfg(lib_dir));

	element = strtok(path, ":");
	while (element) {
		buf = malloc(strlen(name) + strlen(element) + 2);
		if (!buf) return(NULL);
		sprintf(buf, "%s/%s", element, name);
		fp = fopen(buf, amode);
		free(buf);
		if (fp)
			break;
		element = strtok(NULL, ":");
	}
	free(path);

	return(fp);
}


void
OnQuit(void (*fn)())
{
    struct on_quit_rec *p;

	p = (struct on_quit_rec *)malloc(sizeof (struct on_quit_rec));
	if (p) {
		p->f = fn;
		p->next = on_quit_list;
		on_quit_list = p;
	} else {
		simcoupe_mesg(SIMCOUPE_FATAL, "malloc failed in OnQuit()!");
		exit(1);
	}

}


/* Shut down the emulator cleanly, calling any functions on the Quit
 * list.  Currently only (vga/x)events.c:closedown().
 */
void
Quit(int status)
{
	struct on_quit_rec *p = on_quit_list;

	while (p) {
		p->f();
		p = p->next;
	}

	textattr(0x1e);
        cprintf(" SimCoup 0.781a ");
	textattr(8);
	cprintf(" AleyKeprt@bigfoot.com");
	textattr(7);
	printf("\nThis executable is " KEYBMODE " keyboard version\n\nExiting....\n");
	exit(status);
}


/* Report an info, warning, error or fatal message.  Exit if a
 * fatal message has been reported.
 */
#ifdef nec_ews
/* NEC's C compiler is broken - we have to use the old varargs method */
void 
simcoupe_mesg(va_alist)
va_dcl
{
	va_list args;
	int error_level;
	char *fmt;

	va_start(args);
	error_level = (int)va_arg(args, int);
	fmt = (char *)va_arg(args, char *);
#else
void
simcoupe_mesg(int error_level, ...)
{
	va_list args;
	char *fmt;

	va_start(args, error_level);
	fmt = (char *)va_arg(args, char *);
#endif
	if (logfp != NULL) {
	
		fprintf(logfp, "SimCoupe: ");
		if (error_level == SIMCOUPE_WARN)
			fprintf(logfp, "warning: ");
		else if (error_level == SIMCOUPE_ERR)
			fprintf(logfp, "error: ");
		else if (error_level == SIMCOUPE_INFO)
			fprintf(logfp, "information: ");
		else if (error_level == SIMCOUPE_FATAL)
			fprintf(logfp, "fatal: ");
	
		vfprintf(logfp, fmt, args);
		fputc('\n', logfp);
	}
	if (error_level == SIMCOUPE_FATAL)
	Quit(1);
}

/* Log file utilities */

int
open_log(void)
{

	if ((logfp = fopen(GetCfg(logfile), "w")) == NULL) {
		simcoupe_mesg(SIMCOUPE_ERR, "can't open log file");
		return -1;
	}	
	return 0;
} 

void
close_log(void)
{

	fclose(logfp);	

}
/* Emulator timing functions - from xz80 (util.c) by Ian Collier */

void startwatch(int flag)
/* int flag;       0 -> reset stopwatch; 1 -> carry on */
{
   if(!flag)millisec=0;
   gettimeofday(&watch,(struct timezone *)0);
   stopped=0;
}

unsigned long stopwatch(){
   unsigned long oldtime;
   unsigned long newtime;
   if(stopped)return millisec;
   oldtime=watch.tv_sec*1000+watch.tv_usec/1000;
   gettimeofday(&watch,(struct timezone *)0);
   newtime=watch.tv_sec*1000+watch.tv_usec/1000;
   stopped=1;
   return millisec+=(newtime-oldtime);
}




/* Implementations of strcasecmp and strncasecmp, if you don't have them */
#ifdef NEED_STRCASECMP
int strcasecmp(char *s1, char *s2)
{
	char c1, c2;

	for ( ; *s1 && *s2; s1++, s2++) {
		c1 = isupper(*s1) ? tolower(*s1) : *s1;
		c2 = isupper(*s2) ? tolower(*s2) : *s2;
		if (c1 != c2) return (c1 - c2);
	}
	if (*s1 || *s2) return (*s1 - *s2);
	return 0;
}

int strncasecmp(char *s1, char *s2, int n)
{
	int i;
	char c1, c2;

	for (i = 0; *s1 && *s2 && i < n; s1++, s2++, i++) {
		c1 = isupper(*s1) ? tolower(*s1) : *s1;
		c2 = isupper(*s2) ? tolower(*s2) : *s2;
		if (c1 != c2) return (c1 - c2);
	}
	if ((i < n) && (*s1 || *s2)) return (*s1 - *s2);
	return 0;
}
#endif
